#!/usr/bin/env python
# vim: set ts=4 sw=4 st=4 expandtab :

import time

startTime = time.time()

import datetime
import sys
import IndexedRedis
from IndexedRedis import IndexedRedisModel, IRField, setDefaultRedisConnectionParams
from IndexedRedis.fields import IRCompressedField, IRFieldChain, IRRawField, IRBytesField, IRUnicodeField

# vim: set ts=8 sw=8 st=8 expandtab :

#  Change this to match your Redis server info
REDIS_CONNECTION_PARAMS = { 'host' : '127.0.0.1', 'port' : 6379, 'db' : 0 }

# Define the model
class Song(IndexedRedisModel):
    
    FIELDS = [ \
            IRField('artist'),
            IRField('title'),
            IRField('album'),
            IRField('track_number', valueType=int), # Convert automatically to/from int
            IRField('duration'),
            IRField('releaseDate', valueType=datetime.datetime),
            IRField('description'),
            IRField('copyright'),
            IRRawField('mp3_data'), # Do not perform any conversion on the data.
            IRFieldChain('thumbnail', [IRBytesField(), IRCompressedField(compressMode='gzip')]),      # Compress this field in storage using "gzip" compression
            IRField('tags', valueType=list),
            IRFieldChain('lyrics', [ IRUnicodeField(encoding='utf-8'), IRCompressedField() ], defaultValue='No lyrics found'),
    ]

    INDEXED_FIELDS = [ \
                'artist',
                'title',
                'track_number',
    ]

    KEY_NAME = 'Songs'

if __name__ == '__main__':

    setDefaultRedisConnectionParams(REDIS_CONNECTION_PARAMS)

    fakeMp3 = b"\x99\x12\x14"

    fakeThumbnail = b"\x15\x1A\x1A\x1A\x1A\x1A\x1A\x1B\x1A\x1A\x1A\x1B" + (b"\x1A" * 30) # Compressable data

    sys.stdout.write('Testing IndexedRedis version %s\n' %(IndexedRedis.__version__,))

    if '--keep-data' not in sys.argv:
            newSongs = []
            # Add data
            songObj = Song(artist='The Merry Men',
                            title='Happy Go Lucky',
                            album='The Merry Men LP',
                            track_number=1,
                            duration='1:58',
                            description='A song about happy people',
                            mp3_data=fakeMp3,
                            thumbnail=fakeThumbnail,
                            tags=['happy', 'guitar', 'smooth'],
                            copyright='Copyright 2012 (c) Media Mogul Incorporated')

            songObj.lyrics = '''To love, to lie,
to feel something rather than die
A line, a fall
I guess we can't be loved by them all..
'''

            newSongs.append(songObj)

            songObj = Song(artist='The Merry Men',
                            title='Joy to Joy',
                            album='The Merry Men LP',
                            track_number=2,
                            duration='2:54',
                            description='A song about joy',
                            mp3_data=fakeMp3,
                            thumbnail=fakeThumbnail,
                            tags=['joyful', 'piano', 'Key C'],
                            copyright='Copyright 2012 (c) Media Mogul Incorporated')

            newSongs.append(songObj)

            songObj = Song(artist='The Unhappy Folk',
                            title='Sadly she waits',
                            album='Misery loses comfort',
                            track_number=1,
                            duration='15:44',
                            description='A sad song',
                            mp3_data=fakeMp3,
                            thumbnail=fakeThumbnail,
                            copyright='Copyright 2014 (c) Cheese Industries')
            newSongs.append(songObj)

            # Atomically reset the Song dataset to just the songs in "newSongs" list
            Song.reset(newSongs)

            # Add some additional songs one-at-a-time
            songObj = Song(artist='Mega Men',
                            title='Nintendo 1',
                            album='Super Tracks',
                            track_number=1,
                            duration='1:15',
                            description='Super Nintendo',
                            mp3_data=fakeMp3,
                            thumbnail=fakeThumbnail,
                            copyright='Copyright 2014 (c) Cheese Industries')
            songObj.save()

            songObj = Song(artist='Mega Men',
                            title='Nintendo 2',
                            album='Super Tracks',
                            track_number=2,
                            duration='1:55',
                            description='Super Nintendo',
                            mp3_data=fakeMp3,
                            thumbnail=fakeThumbnail,
                            copyright='Copyright 2014 (c) Cheese Industries')
            songObj.save()


    # Query some songs by artist
    merryMenSongs = Song.objects.filter(artist='The Merry Men').all()

    sys.stdout.write("Merry Men Songs:\n")
    merryMenSongs.pprint()

    # Query some songs not by artist
    sys.stdout.write('\n\nNot Mega Men Songs:\n')
    notMegaMenSongs = Song.objects.filterInline(artist__ne='Mega Men').all()
    notMegaMenSongs.pprint()


    sys.stdout.write('\n\n')
    # Show passing filter objects around functions, and not actually fetching until .all is called.
    def getTracks(filterSet, trackNo):
            return filterSet.filter(track_number=trackNo).all()

    sys.stdout.write('\nAll track one songs:\n')

    allTrackOneSongs = getTracks(Song.objects, 1)
    for song in allTrackOneSongs:
            song.pprint()
            sys.stdout.write('\n')

    sys.stdout.write('\nMega Men track ones:')
    megaMenTracks = Song.objects.filter(artist='Mega Men')


    objs = getTracks(megaMenTracks, 1)
    for song in objs:
            song.pprint()
            sys.stdout.write('\n')

    sys.stdout.write('\nMega Men track twos (should be just one entry):\n')

    objs = getTracks(megaMenTracks, 2)
    for song in objs:
            song.pprint()
            sys.stdout.write('\n')

    pk = song.getPk()
    sys.stdout.write('\nPrimary key of previous song: %s\n' %(pk,))

    sys.stdout.write('\nSong pk=%s exists? (should be True) %s\n' %(pk, str(Song.objects.exists(pk)) ) )

    sys.stdout.write('\nAfter Delete, Mega Men Track twos (should be blank):\n')
    song.delete()
    objs = getTracks(megaMenTracks, 2)
    for song in objs:
            song.pprint()
            sys.stdout.write('\n')

    sys.stdout.write('\nSong pk=%s exists? (should now be False) %s\n' %(pk, str(Song.objects.exists(pk)) ) )
    
    # delete remaining Mega Men songs
    numDeleted = Song.objects.filter(artist='Mega Men').delete()
    sys.stdout.write('\nDeleting remaining Mega Men Tracks: %d deleted.\n' %(numDeleted,))

    sys.stdout.write('\nRemaining Mega Men tracks (should be blank):\n')
    songs = Song.objects.filter(artist='Mega Men').all()
    sys.stdout.write(str(songs) + '\n')

    endTime = time.time()

    sys.stderr.write("TIME: %.6f\n" %(endTime - startTime, ))
        

# vim: set ts=4 sw=4 st=4 expandtab :



###############################
##         OUTPUT:            #
###############################

#Testing IndexedRedis version 4.1.4
#Merry Men Songs:
#{'album': 'The Merry Men LP',
# 'artist': 'The Merry Men',
# 'copyright': 'Copyright 2012 (c) Media Mogul Incorporated',
# 'description': 'A song about joy',
# 'duration': '2:54',
# 'mp3_data': b'\x99\x12\x14',
# 'releaseDate': IRNullType(),
# 'tags': ['joyful', 'piano', 'Key C'],
# 'thumbnail': b'\x15\x1a\x1a\x1a\x1a\x1a\x1a\x1b\x1a\x1a\x1a\x1b'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a',
# 'title': 'Joy to Joy',
# 'track_number': 2}
#
#{'album': 'The Merry Men LP',
# 'artist': 'The Merry Men',
# 'copyright': 'Copyright 2012 (c) Media Mogul Incorporated',
# 'description': 'A song about happy people',
# 'duration': '1:58',
# 'mp3_data': b'\x99\x12\x14',
# 'releaseDate': IRNullType(),
# 'tags': ['happy', 'guitar', 'smooth'],
# 'thumbnail': b'\x15\x1a\x1a\x1a\x1a\x1a\x1a\x1b\x1a\x1a\x1a\x1b'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a',
# 'title': 'Happy Go Lucky',
# 'track_number': 1}
#
#
#
#Not Mega Men Songs:
#{'album': 'The Merry Men LP',
# 'artist': 'The Merry Men',
# 'copyright': 'Copyright 2012 (c) Media Mogul Incorporated',
# 'description': 'A song about joy',
# 'duration': '2:54',
# 'mp3_data': b'\x99\x12\x14',
# 'releaseDate': IRNullType(),
# 'tags': ['joyful', 'piano', 'Key C'],
# 'thumbnail': b'\x15\x1a\x1a\x1a\x1a\x1a\x1a\x1b\x1a\x1a\x1a\x1b'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a',
# 'title': 'Joy to Joy',
# 'track_number': 2}
#
#{'album': 'Misery loses comfort',
# 'artist': 'The Unhappy Folk',
# 'copyright': 'Copyright 2014 (c) Cheese Industries',
# 'description': 'A sad song',
# 'duration': '15:44',
# 'mp3_data': b'\x99\x12\x14',
# 'releaseDate': IRNullType(),
# 'tags': IRNullType(),
# 'thumbnail': b'\x15\x1a\x1a\x1a\x1a\x1a\x1a\x1b\x1a\x1a\x1a\x1b'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a',
# 'title': 'Sadly she waits',
# 'track_number': 1}
#
#{'album': 'The Merry Men LP',
# 'artist': 'The Merry Men',
# 'copyright': 'Copyright 2012 (c) Media Mogul Incorporated',
# 'description': 'A song about happy people',
# 'duration': '1:58',
# 'mp3_data': b'\x99\x12\x14',
# 'releaseDate': IRNullType(),
# 'tags': ['happy', 'guitar', 'smooth'],
# 'thumbnail': b'\x15\x1a\x1a\x1a\x1a\x1a\x1a\x1b\x1a\x1a\x1a\x1b'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a',
# 'title': 'Happy Go Lucky',
# 'track_number': 1}
#
#
#
#
#All track one songs:
#{'album': 'Super Tracks',
# 'artist': 'Mega Men',
# 'copyright': 'Copyright 2014 (c) Cheese Industries',
# 'description': 'Super Nintendo',
# 'duration': '1:15',
# 'mp3_data': b'\x99\x12\x14',
# 'releaseDate': IRNullType(),
# 'tags': IRNullType(),
# 'thumbnail': b'\x15\x1a\x1a\x1a\x1a\x1a\x1a\x1b\x1a\x1a\x1a\x1b'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a',
# 'title': 'Nintendo 1',
# 'track_number': 1}
#
#{'album': 'Misery loses comfort',
# 'artist': 'The Unhappy Folk',
# 'copyright': 'Copyright 2014 (c) Cheese Industries',
# 'description': 'A sad song',
# 'duration': '15:44',
# 'mp3_data': b'\x99\x12\x14',
# 'releaseDate': IRNullType(),
# 'tags': IRNullType(),
# 'thumbnail': b'\x15\x1a\x1a\x1a\x1a\x1a\x1a\x1b\x1a\x1a\x1a\x1b'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a',
# 'title': 'Sadly she waits',
# 'track_number': 1}
#
#{'album': 'The Merry Men LP',
# 'artist': 'The Merry Men',
# 'copyright': 'Copyright 2012 (c) Media Mogul Incorporated',
# 'description': 'A song about happy people',
# 'duration': '1:58',
# 'mp3_data': b'\x99\x12\x14',
# 'releaseDate': IRNullType(),
# 'tags': ['happy', 'guitar', 'smooth'],
# 'thumbnail': b'\x15\x1a\x1a\x1a\x1a\x1a\x1a\x1b\x1a\x1a\x1a\x1b'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a',
# 'title': 'Happy Go Lucky',
# 'track_number': 1}
#
#
#Mega Men track ones:{'album': 'Super Tracks',
# 'artist': 'Mega Men',
# 'copyright': 'Copyright 2014 (c) Cheese Industries',
# 'description': 'Super Nintendo',
# 'duration': '1:15',
# 'mp3_data': b'\x99\x12\x14',
# 'releaseDate': IRNullType(),
# 'tags': IRNullType(),
# 'thumbnail': b'\x15\x1a\x1a\x1a\x1a\x1a\x1a\x1b\x1a\x1a\x1a\x1b'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a',
# 'title': 'Nintendo 1',
# 'track_number': 1}
#
#
#Mega Men track twos (should be just one entry):
#{'album': 'Super Tracks',
# 'artist': 'Mega Men',
# 'copyright': 'Copyright 2014 (c) Cheese Industries',
# 'description': 'Super Nintendo',
# 'duration': '1:55',
# 'mp3_data': b'\x99\x12\x14',
# 'releaseDate': IRNullType(),
# 'tags': IRNullType(),
# 'thumbnail': b'\x15\x1a\x1a\x1a\x1a\x1a\x1a\x1b\x1a\x1a\x1a\x1b'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a\x1a'
#              b'\x1a\x1a\x1a\x1a\x1a\x1a',
# 'title': 'Nintendo 2',
# 'track_number': 2}
#
#
#Primary key of previous song: 6
#
#Song pk=6 exists? (should be True) True
#
#After Delete, Mega Men Track twos (should be blank):
#
#Song pk=6 exists? (should now be False) False
#
#Deleting remaining Mega Men Tracks: 1 deleted.
#
#Remaining Mega Men tracks (should be blank):
#IRQueryableList([])


# vim: set ts=4 sw=4 st=4 expandtab :



# Copyright (c) 2016, 2017 Timothy Savannah under LGPL version 2.1. See LICENSE for more information.
#
# FieldValueTypes - Types that can be passed to "valueType" of an IRField for special implicit conversions
#

import json

from . import irNull, IRNullType

from datetime import datetime
from ..compat_str import to_unicode

__all__ = ('IRDatetimeValue', 'IRJsonValue')

try:
    unicode
except NameError:
    unicode = str

class IRDatetimeValue(datetime):
    '''
        IRDatetimeValue - A field type that is a datetime. Pass this as "valueType" to an IRField to use a datetime
    '''

    CAN_INDEX = True

    def __new__(self, *args, **kwargs):
        # No need to actually create this object, we just need to override instantiation to create a datetime from several forms.
        if len(args) == 1:
            if type(args[0]) == bytes:
                theArg = to_unicode(args[0])
            else:
                theArg = args[0]

            if type(theArg) in (str, unicode) and (theArg.isdigit() or len([1 for x in theArg.split('.') if x.isdigit()]) == 2):
                tmp = datetime.fromtimestamp(float(theArg))
            elif type(theArg) in (int, float):
                tmp = datetime.fromtimestamp(float(theArg))
            elif issubclass(theArg.__class__, datetime):
                tmp = theArg
            else:
                # If microsecond is present, drop it. Not available on all platforms.
                if '.' in theArg:
                    theArg = theArg[:theArg.index('.')]
                tmp = datetime.strptime(theArg, '%Y-%m-%d %H:%M:%S')
                
            return datetime(tmp.year, tmp.month, tmp.day, tmp.hour, tmp.minute, tmp.second)
        else:
            # If microsecond is present, drop it. Not available on all platforms.
            if 'microsecond' in kwargs:
                kwargs.pop('microsecond')
            elif len(args) == 7:
                args = args[:-1]

            return datetime(*args, **kwargs)

#    def __repr__(self):
#        '''
#            __repr__ - Be invisible.
#        '''
#        return datetime.__repr__(self).replace('IRDatetimeValue', 'datetime')

def __ir_json__str__(self):
#    if bool(self) is False:
#        return irNull

    return json.dumps(self)
        

class _IRJsonDict(dict):
    __str__ = __ir_json__str__
    def __new__(self, *args, **kwargs):
        return dict.__new__(self, *args, **kwargs)
_IRJsonObject = _IRJsonDict

class _IRJsonString(str):
    __str__ = __ir_json__str__
    def __new__(self, *args, **kwargs):
        return str.__new__(self, *args, **kwargs)

# Cannot subclass bool..... how stupid.
#class _IRJsonBool(bool):
#    __str__ = __ir_json__str__
#    def __new__(self, *args, **kwargs):
#        return bool.__new__(self, *args, **kwargs)
    

class _IRJsonArray(list):
    __str__ = __ir_json__str__
    def __new__(self, *args, **kwargs):
        return list.__new__(self, *args, **kwargs)

class _IRJsonNumber(float):
    __str__ = __ir_json__str__
    def __new__(self, *args, **kwargs):
        return float.__new__(self, *args, **kwargs)

# Cannot subclass NoneType, so use IRNullType to represent null.
class _IRJsonNull(IRNullType):
    def __str__(self):
        return 'null'
    

class IRJsonValue(type):
    '''
        IRJsonValue - A value which is interpreted as json. 
            Supports object (dicts), strings, array, number. 

        "null" is supported using IRNullType, because cannot subclass "NoneType".
        "bool" is supported using a number 0.0 or 1.0 - because cannot subclass "bool"
    '''

    # TODO: This generally can't be indexable, because while the outer may be fixed-order for some
    #   types, inner elements (like elements in a list) might not be. So it'd be dependent on the
    #   value itself, not the value type each time, which makes it not 100% yes or no, which means no.
    CAN_INDEX = False

    def __init__(self, *args, **kwargs):
        pass


    def __str__(self):
        if bool(self) is False:
            return irNull

        return json.dumps(self)
            

    def __new__(self, *args, **kwargs):
        if len(args) == 1:


            if issubclass(args[0].__class__, dict):
                myRet = _IRJsonDict()
                myRet.update(args[0])
                return myRet

            elif issubclass(args[0].__class__, (list, tuple)):
                myRet = _IRJsonArray(args[0])
                return myRet
            elif issubclass(args[0].__class__, (int, float)):
                myRet = _IRJsonNumber(args[0])
                return myRet
            elif isinstance(args[0], bool):
                if args[0] is False:
                    val = 0
                else:
                    val = 1
                myRet = _IRJsonNumber(val)
                return myRet
            elif args[0] is None or issubclass(args[0].__class__, IRNullType):
                myRet = _IRJsonNull()
                return myRet
                
            if type(args[0]) == bytes:
                theArg = to_unicode(args[0])
            else:
                theArg = args[0]

            if len(theArg) == 0:
                return irNull

            try:
                jsonObj = json.loads(theArg)
            except Exception as e:
                raise ValueError('Cannot decode json [%s]: %s%s' %(str(e), str(type(theArg)), repr(theArg)))

            if isinstance(jsonObj, dict):
                myRet = _IRJsonDict()
                myRet.update(jsonObj)
                return myRet
            elif issubclass(jsonObj.__class__, (list, tuple, set)):
                myRet = _IRJsonArray(jsonObj)
                return myRet
            elif isinstance(jsonObj, (int, float)):
                myRet = _IRJsonNumber(jsonObj)
                return myRet
            elif isinstance(jsonObj, bool):
                if jsonObj is False:
                    val = 0
                else:
                    val = 1
                myRet = _IRJsonNumber(val)
                return myRet
            elif jsonObj is None:
                return _IRJsonNull()
            else:
                myRet = _IRJsonString(jsonObj)
                return myRet

        return irNull



'''
    TestProperties - Set properties that will be used by the unit tests
'''

REDIS_CONNECTION_PARAMS = { 'host' : '127.0.0.1', 'port' : 6379, 'db' : 0 }

# Use a function to keep namespace clean
def initProperties():

    try:
        from IndexedRedis import setDefaultRedisConnectionParams, getRedisPool
    except:
        raise Exception('Cannot import IndexedRedis. Ensure that it is installed, or use runTests.py to execute using the version in the source dir (non-site install).')

    from redis import Redis


    setDefaultRedisConnectionParams(REDIS_CONNECTION_PARAMS)

    try:
        _pool = getRedisPool(REDIS_CONNECTION_PARAMS)

        r = Redis(connection_pool=_pool)

        r.info()
    except:
        raise Exception('Cannot connect to Redis using: %s. Please set connection info in TestProperties.py and ensure that Redis is running.')

initProperties()



#!/usr/bin/env python

import sys

from setuptools import setup

if __name__ == '__main__':

    summary = 'A super-fast ORM backed by Redis, supporting models and indexes with O(1) searches, and support for storing native/complex types and objects'
    try:
        with open('README.rst', 'rt') as f:
            long_description = f.read()
    except Exception as e:
        sys.stderr.write('Error reading description: %s\n' %(str(e),))
        long_description = summary

    setup(name='indexedredis',
        version='6.0.3',
        packages=['IndexedRedis', 'IndexedRedis.fields'],
        install_requires=['redis', 'QueryableList'],
        requires=['redis', 'QueryableList'],
        provides=['indexedredis'],
        keywords=['redis', 'IndexedRedis', 'SQL', 'nosql', 'orm', 'fast', 'python', 'filter', 'index', 'model'],
        url='https://github.com/kata198/indexedredis',
        long_description=long_description,
        author='Tim Savannah',
        author_email='kata198@gmail.com',
        maintainer='Tim Savannah',
        maintainer_email='kata198@gmail.com',
        license='LGPLv2',
        description=summary,
        classifiers=['Development Status :: 5 - Production/Stable',
            'Programming Language :: Python',
            'License :: OSI Approved :: GNU Lesser General Public License v2 (LGPLv2)',
            'Programming Language :: Python :: 2',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: 3.6',
            'Topic :: Database :: Front-Ends',
            'Topic :: Software Development :: Libraries :: Python Modules',
        ]
        
    )

#vim: set ts=4 sw=4 expandtab




#!/usr/bin/env python

import datetime
import sys
import IndexedRedis
from IndexedRedis import IndexedRedisModel, IRField
from IndexedRedis.fields.FieldValueTypes import IRJsonValue

# vim: ts=4 sw=4 expandtab

class MyJsonModel(IndexedRedisModel):

    FIELDS = [ \
        IRField('name'),
        IRField('data', valueType=IRJsonValue),
    ]

    INDEXED_FIELDS = [ \
        'name',
    ]

    KEY_NAME = 'MyJsonModel'


if __name__ == '__main__':

    MyJsonModel.objects.delete()

    vanillaDict = { 'hello' : 'world', 'deep' : { 'getting' : 'deeper' } }
    
    newObj = MyJsonModel(name='hello', data=vanillaDict)

    print ( "JsonDict data: <%s>%s\n\n" %(str(type(newObj.data)), repr(newObj.data)))
    print ( "JsonDict dict (not for storage): %s\n\n" %(str(newObj.asDict(forStorage=False, strKeys=True)),))
    print ( "JsonDict dict (for storage): %s\n\n" %(str(newObj.asDict(forStorage=True, strKeys=True)),))

    newObj.save()

    fetchDict = MyJsonModel.objects.filter(name='hello').first()

    if not fetchDict:
        print ( "No result for fetchDict\n\n" )
    else:
        print ( "fetchDict data: <%s>%s\n\n" %(str(type(fetchDict.data)), repr(fetchDict.data)))
        print ( "fetchDict dict (not for storage): %s\n\n" %(str(fetchDict.asDict(forStorage=False, strKeys=True)),))
        print ( "fetchDict dict (for storage): %s\n\n" %(str(fetchDict.asDict(forStorage=True, strKeys=True)),))

    myList = [1, 2]

    obj2 = MyJsonModel(name='listy', data=myList)
    print ( "JsonArray data: <%s>%s\n\n" %(str(type(obj2.data)), repr(obj2.data)))
    print ( "JsonArray dict (not for storage): %s\n\n" %(str(obj2.asDict(forStorage=False, strKeys=True)),))
    print ( "JsonArray dict (for storage): %s\n\n" %(str(obj2.asDict(forStorage=True, strKeys=True)),))

    obj2.save()

    fetchArray = MyJsonModel.objects.filter(name='listy').first()
    
    if not fetchArray:
        print ( "No result for fetchArray\n\n" )
    else:
        print ( "fetchArray data: <%s>%s\n\n" %(str(type(fetchArray.data)), repr(fetchArray.data)))
        print ( "fetchArray dict (not for storage): %s\n\n" %(str(fetchArray.asDict(forStorage=False, strKeys=True)),))
        print ( "fetchArray dict (for storage): %s\n\n" %(str(fetchArray.asDict(forStorage=True, strKeys=True)),))

    myStr = '"Free like the mountain stream"'

    obj3 = MyJsonModel(name='stringy', data=myStr)
    
    print ( "JsonString data: <%s>%s\n\n" %(str(type(obj3.data)), repr(obj3.data)))
    print ( "JsonString dict (not for storage): %s\n\n" %(str(obj3.asDict(forStorage=False, strKeys=True)),))
    print ( "JsonString dict (for storage): %s\n\n" %(str(obj3.asDict(forStorage=True, strKeys=True)),))

    obj3.save()

    fetchString = MyJsonModel.objects.filter(name='stringy').first()
    
    if not fetchString:
        print ( "No result for fetchString\n\n" )
    else:
        print ( "fetchString data: <%s>%s\n\n" %(str(type(fetchString.data)), repr(fetchString.data)))
        print ( "fetchString dict (not for storage): %s\n\n" %(str(fetchString.asDict(forStorage=False, strKeys=True)),))
        print ( "fetchString dict (for storage): %s\n\n" %(str(fetchString.asDict(forStorage=True, strKeys=True)),))

    obj4 = MyJsonModel(name='inty', data=14)
    
    print ( "JsonInt data: <%s>%s\n\n" %(str(type(obj4.data)), repr(obj4.data)))
    print ( "JsonInt dict (not for storage): %s\n\n" %(str(obj4.asDict(forStorage=False, strKeys=True)),))
    print ( "JsonInt dict (for storage): %s\n\n" %(str(obj4.asDict(forStorage=True, strKeys=True)),))

    obj4.save()

    fetchInt = MyJsonModel.objects.filter(name='inty').first()
    
    if not fetchInt:
        print ( "No result for fetchInt\n\n" )
    else:
        print ( "fetchInt data: <%s>%s\n\n" %(str(type(fetchInt.data)), repr(fetchInt.data)))
        print ( "fetchInt dict (not for storage): %s\n\n" %(str(fetchInt.asDict(forStorage=False, strKeys=True)),))
        print ( "fetchInt dict (for storage): %s\n\n" %(str(fetchInt.asDict(forStorage=True, strKeys=True)),))

    obj5 = MyJsonModel(name='booly', data=False)
    
    print ( "JsonBool data: <%s>%s\n\n" %(str(type(obj5.data)), repr(obj5.data)))
    print ( "JsonBool dict (not for storage): %s\n\n" %(str(obj5.asDict(forStorage=False, strKeys=True)),))
    print ( "JsonBool dict (for storage): %s\n\n" %(str(obj5.asDict(forStorage=True, strKeys=True)),))

    obj5.save()

    fetchBool = MyJsonModel.objects.filter(name='booly').first()
    
    if not fetchBool:
        print ( "No result for fetchBool\n\n" )
    else:
        print ( "fetchBool data: <%s>%s\n\n" %(str(type(fetchBool.data)), repr(fetchBool.data)))
        print ( "fetchBool dict (not for storage): %s\n\n" %(str(fetchBool.asDict(forStorage=False, strKeys=True)),))
        print ( "fetchBool dict (for storage): %s\n\n" %(str(fetchBool.asDict(forStorage=True, strKeys=True)),))


    obj6 = MyJsonModel(name='novaluey')
    print ( "JsonNoValue data: <%s>%s\n\n" %(str(type(obj6.data)), repr(obj6.data)))
    print ( "JsonNoValue dict (not for storage): %s\n\n" %(str(obj6.asDict(forStorage=False, strKeys=True)),))
    print ( "JsonNoValue dict (for storage): %s\n\n" %(str(obj6.asDict(forStorage=True, strKeys=True)),))

    obj6.save()

    fetchNoValue = MyJsonModel.objects.filter(name='novaluey').first()
    
    if not fetchNoValue:
        print ( "No result for fetchNoValue\n\n" )
    else:
        print ( "fetchNoValue data: <%s>%s\n\n" %(str(type(fetchNoValue.data)), repr(fetchNoValue.data)))
        print ( "fetchNoValue dict (not for storage): %s\n\n" %(str(fetchNoValue.asDict(forStorage=False, strKeys=True)),))
        print ( "fetchNoValue dict (for storage): %s\n\n" %(str(fetchNoValue.asDict(forStorage=True, strKeys=True)),))

    obj7 = MyJsonModel(name='nully', data=None)
    print ( "JsonNull data: <%s>%s\n\n" %(str(type(obj7.data)), repr(obj7.data)))
    print ( "JsonNull dict (not for storage): %s\n\n" %(str(obj7.asDict(forStorage=False, strKeys=True)),))
    print ( "JsonNull dict (for storage): %s\n\n" %(str(obj7.asDict(forStorage=True, strKeys=True)),))

    obj7.save()

    fetchNull = MyJsonModel.objects.filter(name='nully').first()
    
    if not fetchNull:
        print ( "No result for fetchNull\n\n" )
    else:
        print ( "fetchNull data: <%s>%s\n\n" %(str(type(fetchNull.data)), repr(fetchNull.data)))
        print ( "fetchNull dict (not for storage): %s\n\n" %(str(fetchNull.asDict(forStorage=False, strKeys=True)),))
        print ( "fetchNull dict (for storage): %s\n\n" %(str(fetchNull.asDict(forStorage=True, strKeys=True)),))
        
        
#    fetch2 = MyJsonModel.objects.filter(data=datetime.datetime(1989, 6, 28, 12, 12, 0)).first()
#    if not fetch2:
#        print ( "No result for fetch2\n\n" )
#    else:
#        print ( "fetch2 timestamp: <%s>%s\n\n" %(str(type(fetch2.timestamp)), repr(fetch2.timestamp)))
#        print ( "fetch2 dict (not for storage): %s\n\n" %(str(fetch2.asDict(forStorage=False, strKeys=True)),))
#        print ( "fetch2 dict (for storage): %s\n\n" %(str(fetch2.asDict(forStorage=True, strKeys=True)),))
        
#    fetch2 = MyJsonModel.objects.filter(data=datetime.datetime(1989, 6, 28, 12, 12, 0)).first()
#    if not fetch2:
#        print ( "No result for fetch2\n\n" )
#    else:
#        print ( "fetch2 timestamp: <%s>%s\n\n" %(str(type(fetch2.timestamp)), repr(fetch2.timestamp)))
#        print ( "fetch2 dict (not for storage): %s\n\n" %(str(fetch2.asDict(forStorage=False, strKeys=True)),))
#        print ( "fetch2 dict (for storage): %s\n\n" %(str(fetch2.asDict(forStorage=True, strKeys=True)),))



#!/usr/bin/env python

# Copyright (c) 2017 Timothy Savannah under LGPL version 2.1. See LICENSE for more information.
#
# TestIRBytesField - Test default values for fields
#

# vim: set ts=4 sw=4 st=4 expandtab

# Import and apply the properties (like Redis connection parameters) for this test.
import TestProperties

import sys
import subprocess

from IndexedRedis import IndexedRedisModel, irNull, getDefaultIREncoding, setDefaultIREncoding
from IndexedRedis.compat_str import tobytes
from IndexedRedis.fields import IRBytesField, IRField

# vim: ts=4 sw=4 expandtab

class TestIRBytesField(object):
    '''
        TestIRBytesField - Test IRBytesField
    '''

    KEEP_DATA = False

    def setup_class(self):
        self.defaultIREncoding = getDefaultIREncoding()

    def setup_method(self, testMethod):
        '''
            setup_method - Called before every method. Should set "self.model" to the model needed for the test.
  
            @param testMethod - Instance method of test about to be called.
        '''
        self.model = None


        if testMethod in (self.test_general, ):
            class Model_BytesValue(IndexedRedisModel):
                
                FIELDS = [
                    IRField('name'),
                    IRBytesField('value', defaultValue=irNull),
                ]

                INDEXED_FIELDS = ['name']

                KEY_NAME='TestIRBytesField__ModelBytesValue'

            self.model = Model_BytesValue
        elif testMethod in (self.test_defaultValue, ):
            class Model_BytesDefaultValue(IndexedRedisModel):
                FIELDS = [
                    IRField('name'),
                    IRBytesField('value', defaultValue=b'woobley'),
                ]

                INDEXED_FIELDS = ['name']

                KEY_NAME = 'TestIRBytesField__ModelBytesDefaultValue'

            self.model = Model_BytesDefaultValue

        elif testMethod in (self.test_indexBytes, ):
            class Model_IndexBytes(IndexedRedisModel):
                FIELDS = [
                    IRField('name'),
                    IRBytesField('value', defaultValue=irNull),
                    IRBytesField('value2', defaultValue=b'xxx'),
                ]

                INDEXED_FIELDS = ['name', 'value', 'value2']

                KEY_NAME = 'TestIRBytesField__IndexBytes'

            self.model = Model_IndexBytes

        elif testMethod == self.test_bytesEncoding:
            setDefaultIREncoding('ascii')
            class Model_BytesEncoding(IndexedRedisModel):
                FIELDS = [
                    IRField('name'),
                    IRBytesField('value', encoding='utf-8'),
                ]

                INDEXED_FIELDS = ['name', 'value']

                KEY_NAME = 'TestIRBytesField__BytesEncoding'

            self.model = Model_BytesEncoding

        # If KEEP_DATA is False (debug flag), then delete all objects before so prior test doesn't interfere
        if self.KEEP_DATA is False and self.model:
            self.model.deleter.destroyModel()

    def teardown_method(self, testMethod):
        '''
            teardown_method - Called after every method.

                If self.model is set, will delete all objects relating to that model. To retain objects for debugging, set TestIRField.KEEP_DATA to True.
        '''

        if self.model and self.KEEP_DATA is False:
            self.model.deleter.destroyModel()

        setDefaultIREncoding(self.defaultIREncoding)


    def test_general(self):
        
        Model = self.model

        obj = Model()

        updatedFields = obj.getUpdatedFields()

        helloWorldBytes = b'\x01Hello World\x01'

        assert updatedFields == {} , 'Expected no updated fields when object is first created.\nExpected: %s\nGot:     %s' %(repr([]), repr(updatedFields) )

        assert obj.value == irNull , 'Expected default value of IRBytesField to be irNull when defaultValue=irNull'

        obj.name = 'one'

        obj.save()

        assert obj.getUpdatedFields() == {} , 'Expected no updated fields after object is saved'

        obj.value = u'Goodbye'

        assert obj.value == b'Goodbye' , 'Expected IRBytesField value to be b"goodbye" after setting to "goodbye" (converted to bytes)'

        obj.value = helloWorldBytes

        assert obj.value == helloWorldBytes , 'Expected IRBytesField value to be b"\\x01Hello World\\x01" after setting to "\\x01Hello World\\x01"'

        
        try:
            dictConverted = obj.asDict(forStorage=False, strKeys=True)
            dictForStorage = obj.asDict(forStorage=True, strKeys=True)
        except Exception as e:
            raise AssertionError('Expected to be able to convert to dict for both storage and non-storage. Got exception: %s %s' %(e.__class__.__name__, str(e)))

        assert dictConverted['value'] == helloWorldBytes, 'Expected asDict(forStorage=False) to contain IRBytesField value as bytes string. Got: %s' %(dictConverted['value'], )
        assert dictForStorage['value'] == helloWorldBytes, 'Expected asDict(forStorage=True) to contain IRBytesField value as bytes string. \nExpected: %s\nGot:     %s' %(repr(helloWorldBytes), repr(dictForStorage['value']) )

        updatedFields = obj.getUpdatedFields()

        assert 'value' in updatedFields , 'Expected "value" to show in updated fields after updating'

        assert updatedFields['value'][0] == irNull , 'Expected old value to be irNull in updatedFields. Got: %s' %(repr(updatedFields['value'][0]), )
        assert updatedFields['value'][1] == helloWorldBytes , 'Expected converted value to be new value in updatedFields. Got: %s' %(repr(updatedFields['value'][1]), )

        obj.save()

        updatedFields = obj.getUpdatedFields()

        assert updatedFields == {} , 'Expected updatedFields to be clear after saving.'

        fetchObj = Model.objects.filter(name='one').first()


        assert fetchObj , 'Expected to be able to fetch object on name="one" after saving.'

        obj = fetchObj

        assert obj.value == helloWorldBytes , 'Expected value of fetched to be converted string (As bytes), %s. Got: %s' %(repr(helloWorldBytes), repr(fetchObj.value), )

        updatedFields = obj.getUpdatedFields()

        assert updatedFields == {} , 'Expected updatedFields to be clear after fetching'

        try:
            dictConverted = obj.asDict(forStorage=False, strKeys=True)
            dictForStorage = obj.asDict(forStorage=True, strKeys=True)
        except Exception as e:
            raise AssertionError('Expected to be able to convert to dict for both storage and non-storage. Got exception: %s %s' %(e.__class__.__name__, str(e)))

        assert dictConverted['value'] == helloWorldBytes, 'After fetching, Expected asDict(forStorage=False) to contain IRBytesField value as bytes string. Got: %s' %(dictConverted['value'], )
        assert dictForStorage['value'] == helloWorldBytes, 'After fetching, Expected asDict(forStorage=True) to contain IRBytesField value as bytes string. \nExpected: %s\nGot:     %s' %(repr(helloWorldBytes), repr(dictForStorage['value']) )

        obj.value = b'q123'


        updatedFields = obj.getUpdatedFields()

        assert 'value' in updatedFields , 'Expected "value" to show in updated fields after updating on fetched object'

        assert updatedFields['value'][0] == helloWorldBytes , 'Expected old value to be %s in updatedFields. Got: %s' %(repr(helloWorldBytes), repr(updatedFields['value'][0]), )
        assert updatedFields['value'][1] == b'q123' , 'Expected converted value to be new value in updatedFields. Got: %s' %(repr(updatedFields['value'][1]), )

        try:
            dictConverted = obj.asDict(forStorage=False, strKeys=True)
            dictForStorage = obj.asDict(forStorage=True, strKeys=True)
        except Exception as e:
            raise AssertionError('Expected to be able to convert to dict for both storage and non-storage. Got exception: %s %s' %(e.__class__.__name__, str(e)))

        assert dictConverted['value'] == b'q123', 'After fetching, then updating, Expected asDict(forStorage=False) to contain IRBytesField value as bytes string. Got: %s' %(dictConverted['value'], )
        assert dictForStorage['value'] == b'q123', 'After fetching, then updating, Expected asDict(forStorage=True) to contain IRBytesField value as bytes string.\nExpected: %s\nGot:     %s' %(repr(b'q123'), repr(dictForStorage['value']) )

        
        obj.save()

        updatedFields = obj.getUpdatedFields()

        assert updatedFields == {} , 'Expected updatedFields to be clear after saving'

    def test_defaultValue(self):

        Model = self.model

        obj = Model()

        assert obj.value == b'woobley' , 'Expected defaultValue to be applied to a bytes field.\nExpected: b"woobley"\nGot:     %s' %(repr(obj.value), )

        obj.name = 'test'

        obj.save()

        assert obj.value == b'woobley' , 'Expected defaultValue to remain on a bytes field after saving'

        objFetched = Model.objects.filter(name='test').first()
        assert objFetched , 'Expected to be able to fetch object'

        obj = objFetched

        assert obj.value == b'woobley' , 'Expected defaultValue to remain on a bytes field after fetching'

        obj.value = b'cheesy'

        obj.save()

        objFetched = Model.objects.filter(name='test').first()
        assert objFetched , 'Expected to be able to fetch object'

        obj = objFetched

        assert obj.value == b'cheesy' , 'Expected to be able to change value from default.'

    def test_indexBytes(self):

        Model = self.model

        obj = Model()

        obj.name = 'test'

        ids = obj.save()

        assert ids and ids[0] , 'Failed to save object'

        # Add a "distraction" object
        otherObj = Model()
        otherObj.value = 'qqq'
        otherObj.value2 = 'zzz'

        otherObj.save()

        objFetched = Model.objects.filter(value=irNull).first()

        assert objFetched , 'Failed to fetch an object with default value (irNull) on a bytes field'

        d1 = obj.asDict(includeMeta=True, forStorage=False, strKeys=True)
        d2 = objFetched.asDict(includeMeta=True, forStorage=False, strKeys=True)

        assert objFetched == obj , 'Object fetched was not expected object.\nExpected: %s\n\nGot:     %s\n' %( 
                obj.asDict(includeMeta=True, forStorage=False, strKeys=True),
                objFetched.asDict(includeMeta=True, forStorage=False, strKeys=True)
                )

        objFetched = Model.objects.filter(value2=b'zzz').first()

        assert objFetched , 'Failed to fetch object using a non-default value on bytes field'

        assert objFetched == otherObj , 'Object fetched was not expected object.\nExpected: %s\n\nGot:     %s\n' %( 
                otherObj.asDict(includeMeta=True, forStorage=False, strKeys=True),
                objFetched.asDict(includeMeta=True, forStorage=False, strKeys=True)
                )

        objFetched = Model.objects.filter(value=b'qqq').first()

        assert objFetched , 'Failed to fetch object using a non-default value on bytes field'

        assert objFetched == otherObj , 'Object fetched was not expected object.\nExpected: %s\n\nGot:     %s\n' %( 
                otherObj.asDict(includeMeta=True, forStorage=False, strKeys=True),
                objFetched.asDict(includeMeta=True, forStorage=False, strKeys=True)
                )

    def test_bytesEncoding(self):

        Model = self.model

        obj = Model()

        obj.name = 'one'

        prettyPicturesUtf8Bytes = b' \xe2\x9c\x8f \xe2\x9c\x90 \xe2\x9c\x91 \xe2\x9c\x92 \xe2\x9c\x93 \xe2\x9c\x94 \xe2\x9c\x95 \xe2\x9c\x96 \xe2\x9c\x97 \xe2\x9c\x98 \xe2\x9c\x99 \xe2\x9c\x9a \xe2\x9c\x9b \xe2\x9c\x9c \xe2\x9c\x9d \xe2\x9c\x9e \xe2\x9c\x9f \xe2\x9c\xa0 \xe2\x9c\xa1 \xe2\x9c\xa2 \xe2\x9c\xa3 \xe2\x9c\xa4 \xe2\x9c\xa5 \xe2\x9c\xa6 \xe2\x9c\xa7 \xe2\x9c\xa9 \xe2\x9c\xaa \xe2\x9c\xab '

        prettyPicturesUtf8 = prettyPicturesUtf8Bytes.decode('utf-8')


        obj.value = prettyPicturesUtf8

        assert obj.value == prettyPicturesUtf8Bytes , 'Expected utf-8 string to be converted to bytes after setting'

        ids = obj.save()

        assert ids and ids[0] , 'Failed to save object'

        fetchedObj = Model.objects.filter(value=prettyPicturesUtf8Bytes).first()

        assert fetchedObj , 'Failed to fetch object by bytes value'


        fetchedObj = Model.objects.filter(value=prettyPicturesUtf8).first()

        assert fetchedObj , 'Failed to fetch object by unicode value'

        obj = fetchedObj

        assert obj.value == prettyPicturesUtf8Bytes , 'Expected after fetch that data is bytes version of utf-8 string'



if __name__ == '__main__':
    sys.exit(subprocess.Popen('GoodTests.py -n1 "%s" %s' %(sys.argv[0], ' '.join(['"%s"' %(arg.replace('"', '\\"'), ) for arg in sys.argv[1:]]) ), shell=True).wait())

# vim: set ts=4 sw=4 expandtab



#!/usr/bin/env python

# Copyright (c) 2016, 2017 Timothy Savannah under LGPL version 2.1. See LICENSE for more information.
#
# test_HashedIndexes - GoodTests unit tests Hashed Indexes
#

# vim: set ts=4 sw=4 expandtab

# Import and apply the properties (like Redis connection parameters) for this test.
import TestProperties

import re
import sys
import subprocess

from IndexedRedis import IndexedRedisModel, IRField


# TODO: Add test for nulls and hashed indexes, and various other object types.
#   Right now IRField directly is the only way to get a hashed index, but may open that up.

class TestHashedIndexes(object):
    '''
        TestHashedIndexes - Test some basic IRField stuff
    '''

    KEEP_DATA = False

    def __init__(self):
        self.models = []

        MD5_DIGEST_LENGTH = 32

        self.hashRE = re.compile("^[a-fA-F0-9]{%d}$" %(MD5_DIGEST_LENGTH,))

    
    def _isHashed(self, val):
        return bool(self.hashRE.match(val))


    def teardown_method(self, testMethod):
        if self.KEEP_DATA is False:
            for model in self.models:
                try:
                    model.deleter.destroyModel()
                except Exception as e:
                    sys.stderr.write('Warning: Error deleting all objects belonging to %s (%s):\n%s%s\n' %(model.__class__.__name__, model.KEY_NAME, str(type(e)), str(e)) )

        self.models = []

    def test_simpleIndexedSetAndFetch(self):
        class HashedIndexMdl1(IndexedRedisModel):
            FIELDS = [ IRField('name'), IRField('value', hashIndex=True) ]

            INDEXED_FIELDS = ['name', 'value']

            KEY_NAME = 'Test_HashedIndexMdl1'

        self.models.append(HashedIndexMdl1)

        myObj = HashedIndexMdl1(name='Tim', value='purple')
        ids = myObj.save()

        assert ids, 'Expected to get ids returned from save, but did not.'
        assert ids[0] == myObj._id , 'Expected id to be set on saved object, but was not.'

        # Test that we did hash the index, and the value remains unchanged.
        myObjDictForStorage = myObj.asDict(forStorage=True)
        assert myObjDictForStorage['value'] == myObj.value, 'Expected hashed value for storage to match direct-access on object. Storage=%s%s direct=%s%s' %(str(type(myObjDictForStorage['value'])), str(myObjDictForStorage['value']), str(type(myObj.value)), str(myObj.value))
        
        assert not self._isHashed(myObj.value) , 'Expected value to not be hashed directly on the object.'
        assert not self._isHashed(myObjDictForStorage['value']) , 'Expected value not to be hashed in storage dict'

        # XXX: Remember, has to pass the complex field, not just the type name. 
        #   Maybe the function should support lookups, but it's marked "internal"
        #   so I think better to keep that assumption optimized
#        indexValue = myObj.objects._get_key_for_index('value', myObj.value)
        indexValue = myObj.objects._get_key_for_index(HashedIndexMdl1.FIELDS[1], myObj.value)
        assert ':' in indexValue , 'Expected indexed value to have a colon. Got: "%s"' %(indexValue,)

        hashValue = indexValue[indexValue.rfind(':')+1:]
        assert self._isHashed(hashValue) , 'Expected hashIndex=True value to have a hashed index. Got: "%s"' %(hashValue,)

        # Remainder is general test copied from elsewhere, just to ensure it works with hashed index
        myObj2 = HashedIndexMdl1(name='Joe', value='blue')
        ids2 = myObj2.save()

        assert ids2, 'Expected to get ids returned from save, but did not.'
        assert ids2[0] == myObj2._id , 'Expected id to be set on saved object, but was not.'

        fetchObj = HashedIndexMdl1.objects.get(ids[0])
        assert fetchObj, 'Expected to get an object returned for .get on same id as saved.'
        assert fetchObj._id == ids[0], 'ID on fetched object not same as we requested.'

        assert fetchObj.name == 'Tim', 'Field "name" does not have correct value. Expected "%s", got "%s"' %('Tim', str(fetchObj.name))
        assert fetchObj.value == 'purple', 'Field "value" does not have correct value'

        fetchObjs2 = HashedIndexMdl1.objects.filter(value='blue').all()

        assert len(fetchObjs2) == 1, 'Expected to get one object back for value="blue", got %d' %(len(fetchObjs2),)
        assert (fetchObjs2[0].name == 'Joe' and fetchObjs2[0].value == 'blue'), 'Got wrong values back on filter'

        assert fetchObj.delete() == 1, 'Expected delete on fetched object to return 1'

        fetchObj = HashedIndexMdl1.objects.get(ids[0])
        assert fetchObj is None, 'Expected object to not be present after deleting, but it was.'


    def test_compat_reindexHashed(self):
        '''
            Test that the compat method does rehash correctly
        '''

        class HashedIdxMdlForReindex(IndexedRedisModel):
            FIELDS = [ IRField('name'), IRField('value', hashIndex=True) ]

            INDEXED_FIELDS = ['name', 'value']

            KEY_NAME = 'Test_HashedIndexMdlForReindex'

        class UnHashedIdxMdlForReindex(IndexedRedisModel):
            FIELDS = [ IRField('name'), IRField('value', hashIndex=False) ]

            INDEXED_FIELDS = ['name', 'value']

            KEY_NAME = 'Test_HashedIndexMdlForReindex'

        self.models.append(HashedIdxMdlForReindex)
        self.models.append(UnHashedIdxMdlForReindex)

        # Test both with fetchAll=True and fetchAll=False (i.e. fetch all beforehand, fetch one-at-a-time.
        for fetchAll in (True, False):
            prefixStr = "compat_convertHashedIndexes(fetchAll=%s) " %(str(fetchAll), )

            HashedIdxMdlForReindex.objects.delete()
            UnHashedIdxMdlForReindex.objects.delete()

            # Start with same model but one has hash, one does not.
            #  First, save the value as a hash
            myObj = HashedIdxMdlForReindex(name='Tim', value='purple')
            ids = myObj.save()

            assert ids , 'Failed to save myObj'


            otherObj = UnHashedIdxMdlForReindex(name='George', value='nurple')
            
            assert otherObj.save() , 'Failed to save otherObj'


            filterResults = HashedIdxMdlForReindex.objects.filter(value='purple').all()
            assert len(filterResults) == 1 , prefixStr + 'Expected to get object off filter, but did not'

            filterResults = UnHashedIdxMdlForReindex.objects.filter(value='purple').all()
            assert len(filterResults) == 0, prefixStr + 'Expected to not get objects without using hash filter'

            filterResults = UnHashedIdxMdlForReindex.objects.filter(name='Tim').all()
            assert len(filterResults) == 1, prefixStr + 'Expected to be able to filter using either model on field that does not change.'

            filterResults = HashedIdxMdlForReindex.objects.filter(name='Tim').all()
            assert len(filterResults) == 1, prefixStr + 'Expected to be able to filter using either model on field that does not change.'

            numGeorge = HashedIdxMdlForReindex.objects.filter(name='George').count()
            assert numGeorge == 1 , prefixStr + 'Expected to get saved object name="George"'

            numNurpleHashed = HashedIdxMdlForReindex.objects.filter(value='nurple').count()
            assert numNurpleHashed == 0 , prefixStr + 'Expected to not be able to fetch unhashed index using hashed index model'

            numNurpleUnhashed = UnHashedIdxMdlForReindex.objects.filter(value='nurple').count()
            assert numNurpleUnhashed == 1 , prefixStr + 'Expected to be able to fetch unhashed index using unhashed model'

            # Now, reindex using the unhashed model. This should flip everythng.
            UnHashedIdxMdlForReindex.objects.compat_convertHashedIndexes(fetchAll)

            filterResults = UnHashedIdxMdlForReindex.objects.filter(value='purple').all()
            assert len(filterResults) == 1, prefixStr + 'Expected to get object after reindexing without hash'

            filterResults = HashedIdxMdlForReindex.objects.filter(value='purple').all()
            assert len(filterResults) == 0, prefixStr + 'Expected to not get object using hash search without hashed index'

            filterResults = UnHashedIdxMdlForReindex.objects.filter(name='Tim').all()
            assert len(filterResults) == 1, prefixStr + 'Expected to be able to filter using either model on field that does not change.'

            filterResults = HashedIdxMdlForReindex.objects.filter(name='Tim').all()
            assert len(filterResults) == 1, prefixStr + 'Expected to be able to filter using either model on field that does not change.'

            numGeorge = HashedIdxMdlForReindex.objects.filter(name='George').count()
            assert numGeorge == 1 , prefixStr + 'Expected to get saved object name="George"'

            numNurpleHashed = HashedIdxMdlForReindex.objects.filter(value='nurple').count()
            assert numNurpleHashed == 0 , prefixStr + 'Expected to not be able to fetch unhashed index using hashed index model'

            numNurpleUnhashed = UnHashedIdxMdlForReindex.objects.filter(value='nurple').count()
            assert numNurpleUnhashed == 1 , prefixStr + 'Expected to be able to fetch unhashed index using unhashed model'

            # Now, flip back to hashed and perform same tests

            HashedIdxMdlForReindex.objects.compat_convertHashedIndexes(fetchAll)

            filterResults = UnHashedIdxMdlForReindex.objects.filter(value='purple').all()
            assert len(filterResults) == 0, prefixStr + 'Expected to not get object after reindexing with hash'

            filterResults = HashedIdxMdlForReindex.objects.filter(value='purple').all()
            assert len(filterResults) == 1, prefixStr + 'Expected to get object using hash search with hashed index'

            filterResults = UnHashedIdxMdlForReindex.objects.filter(name='Tim').all()
            assert len(filterResults) == 1, prefixStr + 'Expected to be able to filter using either model on field that does not change.'

            filterResults = HashedIdxMdlForReindex.objects.filter(name='Tim').all()
            assert len(filterResults) == 1, prefixStr + 'Expected to be able to filter using either model on field that does not change.'


            numGeorge = HashedIdxMdlForReindex.objects.filter(name='George').count()
            assert numGeorge == 1 , prefixStr + 'Expected to get saved object name="George"'

            numNurpleHashed = HashedIdxMdlForReindex.objects.filter(value='nurple').count()
            assert numNurpleHashed == 1 , prefixStr + 'Expected to be able to fetch originally unhashed value after conversion to hashed using hashed model'

            numNurpleUnhashed = UnHashedIdxMdlForReindex.objects.filter(value='nurple').count()
            assert numNurpleUnhashed == 0 , prefixStr + 'Expected to not be able to fetch originally unhashed value after conversion to hashed using unhashed model'


if __name__ == '__main__':
    sys.exit(subprocess.Popen('GoodTests.py -n1 "%s" %s' %(sys.argv[0], ' '.join(['"%s"' %(arg.replace('"', '\\"'), ) for arg in sys.argv[1:]]) ), shell=True).wait())

# vim: set ts=4 sw=4 expandtab



# Copyright (c) 2014, 2015, 2016, 2017 Timothy Savannah under LGPL version 2.1. See LICENSE for more information.
#
# fields.compressed - Some types and objects related to compressed fields. Use in place of IRField ( in FIELDS array to activate functionality )
#

# vim:set ts=8 shiftwidth=8 softtabstop=8 noexpandtab :


from . import IRField
from .null import irNull

from ..compat_str import getDefaultIREncoding, tobytes, to_unicode

# Note for this class, if encoding=None we fetch getDefaultIREncoding on every convert,
#   because some usage scenario somewhere may need to use a global and have it change depending on operation
class IRUnicodeField(IRField):
	'''
		IRUnicodeField - A field which supports storing/retrieving data in an arbitrary unicode encoding (may be different than global getDefaultIREncoding/setDefaultIREncoding)

		If "encoding" is None (default) - each time the value is converted getEncoding() will be called to get the encoding. This makes it roughly the same as
		a regular IRField.

		In practice, you may have fields with different encodings (different languages maybe, different platforms, etc), and through this you can support those cases.

		This field type is indeaxble, and the index is forced to be hashed.
	'''

	CAN_INDEX = True

	# We gotta hash this to ensure it works
	hashIndex = True

	def __init__(self, name='', encoding=None, defaultValue=irNull):
		'''
			__init__ - Create an IRUnicodeField

			@param name <str> - The field name
			
			@param encoding <None/str> - A specific encoding to use. If None, defaultIREncoding will be used.

			@param defaultValue - The default value for this field

			This field type is indeaxble, and the index is forced to be hashed.
		'''
		self.valueType = None
		self.encoding = encoding
		self.defaultValue = defaultValue


	def getEncoding(self):
		'''
			getEncoding - Get the encoding codec associated with this field.

				If you provided None, this will return the defaultIREncoding

			@return <str> - Encoding
		'''
		if not self.encoding:
			return getDefaultIREncoding()
		return self.encoding

	def _fromStorage(self, value):
		return to_unicode(value, encoding=self.getEncoding())

	def _toStorage(self, value):
		return tobytes(value, encoding=self.getEncoding())

	def _fromInput(self, value):
		return to_unicode(value, encoding=self.getEncoding())


	def toBytes(self, value):
		'''
			toBytes - Convert a value to bytes using the encoding specified on this field

			@param value <str> - The field to convert to bytes

			@return <bytes> - The object encoded using the codec specified on this field.

			NOTE: This method may go away.
		'''
		if type(value) == bytes:
			return value
		return value.encode(self.getEncoding())

	def _getReprProperties(self):
		return ['encoding=%s' %(repr(self.encoding), )]

	def copy(self):
		return self.__class__(name=self.name, encoding=self.encoding, defaultValue=self.defaultValue)

	def __new__(self, name='', encoding=None, defaultValue=irNull):
		return IRField.__new__(self, name)

# vim:set ts=8 shiftwidth=8 softtabstop=8 noexpandtab :



#!/usr/bin/env python
# -*- coding: utf-8, utf-16 -*-

# ^ The above line is a hack for python2 with its poor unicode implementation to even support compiling this test.
#   So long as the utf-16 strings are also utf-u compatible (albeit nonsense), it will work with python2.
#
#  In this test we set the defaultIREncoding to "ascii" to make sure we are actually working with the data correctly,
#   and using the correct encoding (i.e. to_unicode would try to encode as ascii by default, so we are sure it is using the provided
#   encoding, and not some default from sys or defaultIREncoding)

# Copyright (c) 2017 Timothy Savannah under LGPL version 2.1. See LICENSE for more information.
#
# TestIRUnicodeField - Test the IRUnicodeField
#

# Import and apply the properties (like Redis connection parameters) for this test.
import TestProperties

# vim: set ts=4 sw=4 st=4 expandtab

import sys
import subprocess

from IndexedRedis import IndexedRedisModel, irNull
from IndexedRedis.compat_str import tobytes, getDefaultIREncoding, setDefaultIREncoding, to_unicode
from IndexedRedis.fields import IRUnicodeField, IRField, IRUnicodeField

# vim: ts=4 sw=4 expandtab

class TestIRUnicodeField(object):
    '''
        TestIRUnicodeField - Test some IRUnicodeField stuff
    '''

    KEEP_DATA = False

    def setup_class(self):

        self.origIREncoding = getDefaultIREncoding()
        setDefaultIREncoding('ascii') # Make sure IRField stuff would normally fail with utf-8 specific codes

        self.prettyPicturesUtf8 = b' \xe2\x9c\x8f \xe2\x9c\x90 \xe2\x9c\x91 \xe2\x9c\x92 \xe2\x9c\x93 \xe2\x9c\x94 \xe2\x9c\x95 \xe2\x9c\x96 \xe2\x9c\x97 \xe2\x9c\x98 \xe2\x9c\x99 \xe2\x9c\x9a \xe2\x9c\x9b \xe2\x9c\x9c \xe2\x9c\x9d \xe2\x9c\x9e \xe2\x9c\x9f \xe2\x9c\xa0 \xe2\x9c\xa1 \xe2\x9c\xa2 \xe2\x9c\xa3 \xe2\x9c\xa4 \xe2\x9c\xa5 \xe2\x9c\xa6 \xe2\x9c\xa7 \xe2\x9c\xa9 \xe2\x9c\xaa \xe2\x9c\xab '

        # Note - just use the bytes value and decode, rather than rely on the syntax parser to do it properly (python2 has issues)

        self.utf16DataBytes = b'\xff\xfe\x01\xd8\x0f\xdc\x01\xd8-\xdc\x01\xd8;\xdc\x01\xd8+\xdc'
        self.utf16Data = self.utf16DataBytes.decode('utf-16')

        self.utf16Data2Bytes = b'\xff\xfe\x01\xd8\x88\xdc\x01\xd8\x9d\xdc\x01\xd8\x91\xdc\x01\xd8\x9b\xdc\x01\xd8\x90\xdc\x01\xd8\x98\xdc\x01\xd8\x95\xdc\x01\xd8\x96\xdc'
        self.utf16Data2 = self.utf16Data2Bytes.decode('utf-16')

    def teardown_class(self):
        
        setDefaultIREncoding(self.origIREncoding)

    def setup_method(self, testMethod):
        '''
            setup_method - Called before every method. Should set "self.model" to the model needed for the test.
  
            @param testMethod - Instance method of test about to be called.
        '''
        self.model = None



        if testMethod in (self.test_general, ):
            class Model_GeneralUnicode(IndexedRedisModel):
                
                FIELDS = [
                    IRField('name'),
                    IRUnicodeField('value', defaultValue=irNull, encoding='utf-8'),
                ]

                INDEXED_FIELDS = ['name']

                KEY_NAME='TestIRUnicodeField__GeneralUnicode'

            self.model = Model_GeneralUnicode
        elif testMethod == self.test_defaultValue:
            class Model_UnicodeDefaultValue(IndexedRedisModel):
                
                FIELDS = [
                    IRField('name'),
                    IRUnicodeField('value', defaultValue=u'qqq', encoding='utf-8'),
                ]

                INDEXED_FIELDS = ['name']

                KEY_NAME = 'TestIRUnicodeField__UnicodeDefaultValue'

            self.model = Model_UnicodeDefaultValue

        elif testMethod == self.test_utf16:
            class Model_Utf16(IndexedRedisModel):
                
                FIELDS = [
                    IRField('name'),
                    IRUnicodeField('value', encoding='utf-16', defaultValue=irNull),
                    IRUnicodeField('value2', encoding='utf-16', defaultValue=''),
                ]

                INDEXED_FIELDS = ['name', 'value']

                KEY_NAME='TestIRUnicodeField__ModelUtf16'

            self.model = Model_Utf16

        # If KEEP_DATA is False (debug flag), then delete all objects before so prior test doesn't interfere
        if self.KEEP_DATA is False and self.model:
            self.model.deleter.destroyModel()

    def teardown_method(self, testMethod):
        '''
            teardown_method - Called after every method.

                If self.model is set, will delete all objects relating to that model. To retain objects for debugging, set TestIRField.KEEP_DATA to True.
        '''
        setDefaultIREncoding(self.origIREncoding) # Revert back to utf-8 encoding

        if self.model and self.KEEP_DATA is False:
            self.model.deleter.destroyModel()


    def test_general(self):
        
        Model = self.model
        prettyPicturesUtf8 = self.prettyPicturesUtf8
        prettyPicturesUtf8Unicode = to_unicode(prettyPicturesUtf8, encoding='utf-8')

        obj = Model()

        updatedFields = obj.getUpdatedFields()

        assert updatedFields == {} , 'Expected no updated fields when object is first created.\nExpected: %s\nGot:     %s' %(repr([]), repr(updatedFields) )

        assert obj.value == irNull , 'Expected default value of IRUnicodeField to be irNull when defaultValue=irNull'

        obj.name = 'one'

        obj.save()

        assert obj.getUpdatedFields() == {} , 'Expected no updated fields after object is saved'

        obj.value = prettyPicturesUtf8

        assert obj.value == prettyPicturesUtf8Unicode , 'Expected IRUnicodeField value to be some unicode after setting'

        try:
            dictConverted = obj.asDict(forStorage=False, strKeys=True)
            dictForStorage = obj.asDict(forStorage=True, strKeys=True)
        except Exception as e:
            raise AssertionError('Expected to be able to convert to dict for both storage and non-storage. Got exception: %s %s' %(e.__class__.__name__, str(e)))

        assert dictConverted['value'] == prettyPicturesUtf8Unicode , 'Expected asDict(forStorage=False) to contain IRUnicodeField value as unicode string. Got: %s' %(repr(dictConverted['value']), ) 
        assert dictForStorage['value'] == prettyPicturesUtf8 , 'Expected asDict(forStorage=True) to contain IRUnicodeField that was bytes.\nExpected: %s\nGot:     %s' %(repr(prettyPicturesUtf8), repr(dictForStorage['value']) )

        updatedFields = obj.getUpdatedFields()

        assert 'value' in updatedFields , 'Expected "value" to show in updated fields after updating'

        assert updatedFields['value'][0] == irNull , 'Expected old value to be irNull in updatedFields. Got: %s' %(repr(updatedFields['value'][0]), )
        assert updatedFields['value'][1] == prettyPicturesUtf8Unicode , 'Expected converted value to be new value in updatedFields. Got: %s' %(repr(updatedFields['value'][1]), )

        obj.save()

        updatedFields = obj.getUpdatedFields()

        assert updatedFields == {} , 'Expected updatedFields to be clear after saving.'

        fetchObj = Model.objects.filter(name='one').first()


        assert fetchObj , 'Expected to be able to fetch object on name="one" after saving.'

        obj = fetchObj

        assert obj.value == prettyPicturesUtf8Unicode , 'Expected value of fetched to be unicode string, %s. Got: %s' %(repr(prettyPicturesUtf8Unicode), repr(fetchObj.value), )

        updatedFields = obj.getUpdatedFields()

        assert updatedFields == {} , 'Expected updatedFields to be clear after fetching'

        try:
            dictConverted = obj.asDict(forStorage=False, strKeys=True)
            dictForStorage = obj.asDict(forStorage=True, strKeys=True)
        except Exception as e:
            raise AssertionError('Expected to be able to convert to dict for both storage and non-storage. Got exception: %s %s' %(e.__class__.__name__, str(e)))

        assert dictConverted['value'] == prettyPicturesUtf8Unicode, 'After fetching, Expected asDict(forStorage=False) to contain IRUnicodeField value as unicode string. Got: %s' %(dictConverted['value'], )
        assert dictForStorage['value'] == prettyPicturesUtf8, 'After fetching, Expected asDict(forStorage=True) to contain IRUnicodeField as bytes.\nExpected: %s\nGot:     %s' %(repr(prettyPicturesUtf8), repr(dictForStorage['value']) )

        obj.value = b'q123'


        updatedFields = obj.getUpdatedFields()

        assert 'value' in updatedFields , 'Expected "value" to show in updated fields after updating on fetched object'

        assert updatedFields['value'][0] == prettyPicturesUtf8Unicode , 'Expected old value to be b"Hello World" in updatedFields. Got: %s' %(repr(updatedFields['value'][0]), )
        assert updatedFields['value'][1] == u'q123' , 'Expected converted value to be new value in updatedFields. Got: %s' %(repr(updatedFields['value'][1]), )

        dictConverted = obj.asDict(forStorage=False, strKeys=True)
        dictForStorage = obj.asDict(forStorage=True, strKeys=True)

        assert dictConverted['value'] == u'q123', 'After fetching, then updating, Expected asDict(forStorage=False) to contain IRUnicodeField value as unicode string. Got: %s' %(dictConverted['value'], )
        assert dictForStorage['value'] == b'q123' , 'After fetching, then updating, Expected asDict(forStorage=True) to contain IRUnicodeField as bytes.\nExpected: %s\nGot:     %s' %(repr(u'q123'), repr(dictForStorage['value']) )

        
        obj.save()

        updatedFields = obj.getUpdatedFields()

        assert updatedFields == {} , 'Expected updatedFields to be clear after saving'

    def test_defaultValue(self):

        Model = self.model

        obj = Model()

        assert obj.value == u'qqq' , 'Expected defaultValue to be applied to a unicode field.\nExpected: b"woobley"\nGot:     %s' %(repr(obj.value), )

        obj.name = 'test'

        obj.save()

        assert obj.value == u'qqq' , 'Expected defaultValue to remain on a unicode field after saving'

        objFetched = Model.objects.filter(name='test').first()
        assert objFetched , 'Expected to be able to fetch object'

        obj = objFetched

        assert obj.value == u'qqq' , 'Expected defaultValue to remain on a unicode field after fetching'

        obj.value = 'cheesy'

        obj.save()

        objFetched = Model.objects.filter(name='test').first()
        assert objFetched , 'Expected to be able to fetch object'

        obj = objFetched

        assert obj.value == u'cheesy' , 'Expected to be able to change value from default.'
        

    def test_utf16(self):

        Model = self.model

        obj = Model()

        assert obj.getUpdatedFields() == {} , 'Expected getUpdatedFields to be blank on new object'

        obj.name = 'one'


        assert obj.value == irNull , 'Expected defaultValue=irNull to be honoured'
        assert obj.value2 == u'', 'Expected defaultValue="" to be honoured'

        updatedFields = obj.getUpdatedFields()

        assert list(updatedFields.keys()) == ['name'] , 'Expected just one entry on updatedFields when just "name" has been changed. Got: %s' %(repr(updatedFields), )

        ids = obj.save()

        assert ids and ids[0] , 'Failed to save objhect'

        fetchedObj = Model.objects.filter(name='one').first()

        assert fetchedObj , 'Failed to fetch object'

        obj = fetchedObj

        updatedFields = obj.getUpdatedFields()
        assert updatedFields == {}, 'Expected updatedFields to be empty after fetch'


        assert obj.value == irNull , 'Expected defaultValue=irNull to be retained after no change and save then fetch'
        assert obj.value2 == u'', 'Expected defaultValue="" to be retained after no change and save then fetch'

        obj.value = self.utf16Data
        
        obj.save()

        otherObj = Model()

        otherObj.name = 'two'
        otherObj.value = self.utf16Data2Bytes

        assert otherObj.value == self.utf16Data2 , 'Expected utf-16 data provided as bytes to be converted to unicode string'

        otherObj.save()

        fetchedObj = Model.objects.filter(value=self.utf16Data).first()

        assert fetchedObj , 'Failed to fetch object on utf-16 data'

        obj = fetchedObj

        assert obj.name == 'one' , 'Fetched wrong object on utf-16 data'

        assert obj.value == self.utf16Data , 'Expected fetched object to contain the correct data'

        assert obj.value.encode('utf-16') == self.utf16DataBytes , 'Expected data to be encodable back into bytes'



if __name__ == '__main__':
    sys.exit(subprocess.Popen('GoodTests.py -n1 "%s" %s' %(sys.argv[0], ' '.join(['"%s"' %(arg.replace('"', '\\"'), ) for arg in sys.argv[1:]]) ), shell=True).wait())

# vim: set ts=4 sw=4 expandtab
